/*
 * Software License Agreement (Modified BSD License)
 *
 *  Copyright (c) 2013, PAL Robotics, S.L.
 *  All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *   * Neither the name of PAL Robotics, S.L. nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 *  COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 *  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 *  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 *  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */

/** \author Paul Mathieu. */

#ifndef CONSTROLLERUPDATER_H
#define CONSTROLLERUPDATER_H

#include <string>
#include <map>
#include <boost/function.hpp>
#include <boost/thread.hpp>
#include <ros/ros.h>
#include <ros/timer.h>

#include "play_motion/datatypes.h"

namespace play_motion
{

  /** Keeps track of controller statuses by polling the controller manager.
 * The service call happens in a separate thread to not disrupt the main code.
 */
  class ControllerUpdater
  {
  public:
    enum ControllerState
    {
      RUNNING,
      STOPPED
    };

    typedef std::map<std::string, ControllerState> ControllerStates;
    typedef std::map<std::string, JointNames>      ControllerJoints;

  private:
    typedef boost::function<void(const ControllerStates& states,
                                 const ControllerJoints& joints)> Callback;

  public:
    ControllerUpdater(ros::NodeHandle nh);
    virtual ~ControllerUpdater();

    void registerUpdateCb(const Callback& cb) { update_cb_ = cb; }

  private:
    void mainLoop();

    ros::NodeHandle    nh_;
    ros::Timer         update_timer_;
    boost::thread      main_thread_;
    Callback           update_cb_;
    ros::ServiceClient cm_client_;
    ControllerStates   last_cstates_;
  };

}

#endif
